<?php

/**
 * WCFM plugin core
 *
 * Appointments WC Appointments Support
 *
 * @author 		WC Lovers
 * @package 	wcfm/core
 * @version   2.4.0
 */
 
class WCFMu_WCAppointments {
	
	public function __construct() {
    global $WCFM, $WCFMu;
    
    if( $wcfm_is_allow_appointments = apply_filters( 'wcfm_is_allow_appointments' , true ) ) {
			if( WCFMu_Dependencies::wcfm_wc_appointments_active_check() ) {
				if ( current_user_can( 'manage_appointments' ) ) {
					// WCFM Appointments Query Var Filter
					add_filter( 'wcfm_query_vars', array( &$this, 'wca_wcfm_query_vars' ), 90 );
					add_filter( 'wcfm_endpoint_title', array( &$this, 'wca_wcfm_endpoint_title' ), 90, 2 );
					add_action( 'init', array( &$this, 'wca_wcfm_init' ), 90 );
					
					// WCFM Appointments Endpoint Edit
					add_filter( 'wcfm_endpoints_slug', array( $this, 'wca_wcfm_endpoints_slug' ) );
					
					// WCFM Menu Filter
					add_filter( 'wcfm_menus', array( &$this, 'wca_wcfm_menus' ), 90 );
					
					// Appointments Product Type
					add_filter( 'wcfm_product_types', array( &$this, 'wca_product_types' ), 90 );
					
					// Appointments Load WCFMu Scripts
					add_action( 'wcfm_load_scripts', array( &$this, 'wca_load_scripts' ), 90 );
					add_action( 'after_wcfm_load_scripts', array( &$this, 'wca_load_scripts' ), 90 );
					
					// Appointments Load WCFMu Styles
					add_action( 'wcfm_load_styles', array( &$this, 'wca_load_styles' ), 90 );
					add_action( 'after_wcfm_load_styles', array( &$this, 'wca_load_styles' ), 90 );
					
					// Appointments Load WCFMu views
					add_action( 'wcfm_load_views', array( &$this, 'wca_load_views' ), 90 );
					add_action( 'before_wcfm_load_views', array( &$this, 'wca_load_views' ), 90 );
					
					// Appointments Ajax Controllers
					add_action( 'after_wcfm_ajax_controller', array( &$this, 'wca_ajax_controller' ) );
					
					// Appointments General Block
					add_action( 'after_wcfm_products_manage_general', array( &$this, 'wca_product_manage_general' ), 90, 2 );
					
					// Appointment Mark as Confirmed
					add_action( 'wp_ajax_wcfm_appointment_mark_confirm', array( &$this, 'wcfm_appointment_mark_confirm' ) );
					add_action( 'wp_ajax_nopriv_wcfm_appointment_mark_confirm', array( &$this, 'wcfm_appointment_mark_confirm' ) );
					
					// Appointment Status Update
					add_action( 'wp_ajax_wcfm_modify_appointment_status', array( &$this, 'wcfm_modify_appointment_status' ) );
					add_action( 'wp_ajax_nopriv_wcfm_modify_appointment_status', array( &$this, 'wcfm_modify_appointment_status' ) );
					
					add_action( 'woocommerce_before_appointment_object_save', array( &$this, 'wcfm_before_appointment_object_save' ), 10, 2 );
				}
			}
		}
  }
  
  /**
   * WC Appointments Query Var
   */
  function wca_wcfm_query_vars( $query_vars ) {
  	$wcfm_modified_endpoints = (array) get_option( 'wcfm_endpoints' );
  	
		$query_appointments_vars = array(
			'wcfm-appointments-dashboard'       => ! empty( $wcfm_modified_endpoints['wcfm-appointments-dashboard'] ) ? $wcfm_modified_endpoints['wcfm-appointments-dashboard'] : 'wcfm-appointments-dashboard',
			'wcfm-appointments'                 => ! empty( $wcfm_modified_endpoints['wcfm-appointments'] ) ? $wcfm_modified_endpoints['wcfm-appointments'] : 'wcfm-appointments',
			'wcfm-appointments-staffs'       => ! empty( $wcfm_modified_endpoints['wcfm-appointments-staffs'] ) ? $wcfm_modified_endpoints['wcfm-appointments-staffs'] : 'wcfm-appointments-staffs',
			'wcfm-appointments-staffs-manage'=> ! empty( $wcfm_modified_endpoints['wcfm-appointments-staffs-manage'] ) ? $wcfm_modified_endpoints['wcfm-appointments-staffs-manage'] : 'wcfm-appointments-staffs-manage',
			'wcfm-appointments-manual'          => ! empty( $wcfm_modified_endpoints['wcfm-appointments-manual'] ) ? $wcfm_modified_endpoints['wcfm-appointments-manual'] : 'wcfm-appointments-manual',
			'wcfm-appointments-calendar'        => ! empty( $wcfm_modified_endpoints['wcfm-appointments-calendar'] ) ? $wcfm_modified_endpoints['wcfm-appointments-calendar'] : 'wcfm-appointments-calendar',
			'wcfm-appointments-details'         => ! empty( $wcfm_modified_endpoints['wcfm-appointments-details'] ) ? $wcfm_modified_endpoints['wcfm-appointments-details'] : 'wcfm-appointments-details',
		);
		
		$query_vars = array_merge( $query_vars, $query_appointments_vars );
		
		return $query_vars;
  }
  
  /**
   * WC Appointments End Point Title
   */
  function wca_wcfm_endpoint_title( $title, $endpoint ) {
  	
  	switch ( $endpoint ) {
  		case 'wcfm-appointments-dashboard' :
				$title = __( 'Appointments Dashboard', 'wc-frontend-manager-ultimate' );
			break;
			case 'wcfm-appointments' :
				$title = __( 'Appointments List', 'wc-frontend-manager-ultimate' );
			break;
			case 'wcfm-appointments-staffs' :
				$title = __( 'Appointments Staffs', 'wc-frontend-manager-ultimate' );
			break;
			case 'wcfm-appointments-staffs-manage' :
				$title = __( 'Appointments Staffs Manage', 'wc-frontend-manager-ultimate' );
			break;
			case 'wcfm-appointments-manual' :
				$title = __( 'Create Appointments', 'wc-frontend-manager-ultimate' );
			break;
			case 'wcfm-appointments-calendar' :
				$title = __( 'Appointments Calendar', 'wc-frontend-manager-ultimate' );
			break;
			case 'wcfm-appointments-details' :
				$title = sprintf( __( 'Appointments Details #%s', 'wc-frontend-manager-ultimate' ), $wp->query_vars['wcfm-appointments-details'] );
			break;
  	}
  	
  	return $title;
  }
  
  /**
   * WC Appointments Endpoint Intialize
   */
  function wca_wcfm_init() {
  	global $WCFM_Query;
	
		// Intialize WCFM End points
		$WCFM_Query->init_query_vars();
		$WCFM_Query->add_endpoints();
		
		if( !get_option( 'wcfm_updated_end_point_wc_appointments' ) ) {
			// Flush rules after endpoint update
			flush_rewrite_rules();
			update_option( 'wcfm_updated_end_point_wc_appointments', 1 );
		}
  }
  
  /**
	 * WC Appointments Endpoiint Edit
	 */
	function wca_wcfm_endpoints_slug( $endpoints ) {
		
		$appointment_endpoints = array(
													'wcfm-appointments-dashboard'        => 'wcfm-appointments-dashboard',
													'wcfm-appointments'                  => 'wcfm-appointments',
													'wcfm-appointments-staffs'           => 'wcfm-appointments-staffs',
													'wcfm-appointments-staffs-manage'    => 'wcfm-appointments-staffs-manage',
													'wcfm-appointments-manual'    		   => 'wcfm-appointments-manual',
													'wcfm-appointments-calendar'  		   => 'wcfm-appointments-calendar',
													'wcfm-appointments-details'          => 'wcfm-appointments-details'
													);
		
		$endpoints = array_merge( $endpoints, $appointment_endpoints );
		
		return $endpoints;
	}
  
  /**
   * WC Appointments Menu
   */
  function wca_wcfm_menus( $menus ) {
  	global $WCFM;
  	
  	if ( current_user_can( 'manage_appointments' ) ) {
			$menus = array_slice($menus, 0, 3, true) +
													array( 'wcfm-appointments-dashboard' => array(   'label'  => __( 'Appointments', 'woocommerce-appointments'),
																											 'url'     => get_wcfm_appointments_dashboard_url(),
																											 'icon'    => 'calendar-check-o'
																											) )	 +
														array_slice($menus, 3, count($menus) - 3, true) ;
		}
		
  	return $menus;
  }
  
  /**
   * WC Appointments Product Type
   */
  function wca_product_types( $pro_types ) {
  	global $WCFM;
  	if ( current_user_can( 'manage_appointments' ) ) {
  		$pro_types['appointment'] = __( 'Appointable product', 'woocommerce-appointments' );
  	}
  	
  	return $pro_types;
  }
  
	/**
	* WC Appointments Scripts
	*/
  public function wca_load_scripts( $end_point ) {
	  global $WCFM, $WCFMu;
    
	  switch( $end_point ) {
	  	case 'wcfm-products-manage':
				wp_enqueue_script( 'wcfmu_wc_appointments_products_manage_js', $WCFMu->library->js_lib_url . 'wc_appointments/wcfmu-script-wcappointments-products-manage.js', array( 'jquery' ), $WCFMu->version, true );
			break;
			
			case 'wcfm-appointments':
      	$WCFM->library->load_datatable_lib();
	    	wp_enqueue_script( 'wcfmu_appointments_js', $WCFMu->library->js_lib_url . 'wc_appointments/wcfmu-script-wcappointments.js', array('jquery', 'dataTables_js'), $WCFMu->version, true );
	    	
	    	// Screen manager
	    	$wcfm_screen_manager = (array) get_option( 'wcfm_screen_manager' );
	    	$wcfm_screen_manager_data = array();
	    	if( isset( $wcfm_screen_manager['appointment'] ) ) $wcfm_screen_manager_data = $wcfm_screen_manager['appointment'];
	    	wp_localize_script( 'wcfmu_appointments_js', 'wcfm_appointments_screen_manage', $wcfm_screen_manager_data );
      break;
      
      case 'wcfm-appointments-staffs':
      	$WCFM->library->load_datatable_lib();
	    	wp_enqueue_script( 'wcfmu_appointments_staffs_js', $WCFMu->library->js_lib_url . 'wc_appointments/wcfmu-script-wcappointments-staffs.js', array('jquery', 'dataTables_js'), $WCFMu->version, true );
      break;
      
      case 'wcfm-appointments-staffs-manage':
	    	wp_enqueue_script( 'wcfmu_appointments_staffs_manage_js', $WCFMu->library->js_lib_url . 'wc_appointments/wcfmu-script-wcappointments-staffs-manage.js', array('jquery'), $WCFMu->version, true );
	    	// Localized Script
        $wcfm_messages = get_wcfm_staffs_manage_messages();
			  wp_localize_script( 'wcfmu_appointments_staffs_manage_js', 'wcfm_staffs_manage_messages', $wcfm_messages );
      break;
      
      case 'wcfm-appointments-manual':
      	$WCFM->library->load_select2_lib();
	    	wp_enqueue_script( 'wcfmu_appointments_manual_js', $WCFMu->library->js_lib_url . 'wc_appointments/wcfmu-script-wcappointments-manual.js', array('jquery', 'select2_js'), $WCFMu->version, true );
      break;
      
      case 'wcfm-appointments-calendar':
      	$WCFM->library->load_tiptip_lib();
      	$WCFM->library->load_datepicker_lib();
	    	wp_enqueue_script( 'wcfmu_appointments_calendar_js', $WCFMu->library->js_lib_url . 'wc_appointments/wcfmu-script-wcappointments-calendar.js', array('jquery'), $WCFMu->version, true );
      break;
      
      case 'wcfm-appointments-details':
	    	wp_enqueue_script( 'wcfmu_appointments_details_js', $WCFMu->library->js_lib_url . 'wc_appointments/wcfmu-script-wcappointments-details.js', array('jquery'), $WCFMu->version, true );
      break;
	  }
	}
	
	/**
   * WC Appointments Styles
   */
	public function wca_load_styles( $end_point ) {
	  global $WCFM, $WCFMu;
		
	  switch( $end_point ) {
	  	case 'wcfm-products-manage':
				wp_enqueue_style( 'wcfmu_wc_appointments_products_manage_css', $WCFMu->library->css_lib_url . 'wc_appointments/wcfmu-style-wcappointments-products-manage.css', array( ), $WCFMu->version );
			break;
			
			case 'wcfm-appointments-dashboard':
	    	wp_enqueue_style( 'wcfmu_appointments_dashboard_css',  $WCFMu->library->css_lib_url . 'wc_appointments/wcfmu-style-wcappointments-dashboard.css', array(), $WCFMu->version );
		  break;
		  
		  case 'wcfm-appointments':
	    	wp_enqueue_style( 'wcfmu_appointments_css',  $WCFMu->library->css_lib_url . 'wc_appointments/wcfmu-style-wcappointments.css', array(), $WCFMu->version );
		  break;
		  
		  case 'wcfm-appointments-staffs':
	    	wp_enqueue_style( 'wcfmu_appointments_staffs_css',  $WCFMu->library->css_lib_url . 'wc_appointments/wcfmu-style-wcappointments-staffs.css', array(), $WCFMu->version );
		  break;
		  
		  case 'wcfm-appointments-staffs-manage':
		  	wp_enqueue_style( 'collapsible_css',  $WCFM->library->css_lib_url . 'wcfm-style-collapsible.css', array(), $WCFMu->version );
	    	wp_enqueue_style( 'wcfmu_appointments_staffs_manage_css',  $WCFMu->library->css_lib_url . 'wc_appointments/wcfmu-style-wcappointments-staffs-manage.css', array(), $WCFMu->version );
		  break;
		  
		  case 'wcfm-appointments-manual':
		  	wp_enqueue_style( 'collapsible_css',  $WCFM->library->css_lib_url . 'wcfm-style-collapsible.css', array(), $WCFMu->version );
	    	wp_enqueue_style( 'wcfmu_appointments_manual_css',  $WCFMu->library->css_lib_url . 'wc_appointments/wcfmu-style-wcappointments-manual.css', array(), $WCFMu->version );
		  break;
		  
		  case 'wcfm-appointments-calendar':
		  	wp_enqueue_style( 'collapsible_css',  $WCFM->library->css_lib_url . 'wcfm-style-collapsible.css', array(), $WCFMu->version );
	    	wp_enqueue_style( 'wcfmu_appointments_calendar_css',  $WCFMu->library->css_lib_url . 'wc_appointments/wcfmu-style-wcappointments-calendar.css', array(), $WCFMu->version );
		  break;
		  
		  case 'wcfm-appointments-details':
		  	wp_enqueue_style( 'collapsible_css',  $WCFM->library->css_lib_url . 'wcfm-style-collapsible.css', array(), $WCFMu->version );
	    	wp_enqueue_style( 'wcfmu_appointments_details_css',  $WCFMu->library->css_lib_url . 'wc_appointments/wcfmu-style-wcappointments-details.css', array(), $WCFMu->version );
		  break;
	  }
	}
	
	/**
   * WC Appointments Views
   */
  public function wca_load_views( $end_point ) {
	  global $WCFM, $WCFMu;
	  
	  switch( $end_point ) {
	  	case 'wcfm-appointments-dashboard':
        require_once( $WCFMu->library->views_path . 'wc_appointments/wcfmu-view-wcappointments-dashboard.php' );
      break;
      
      case 'wcfm-appointments':
        require_once( $WCFMu->library->views_path . 'wc_appointments/wcfmu-view-wcappointments.php' );
      break;
      
      case 'wcfm-appointments-staffs':
        require_once( $WCFMu->library->views_path . 'wc_appointments/wcfmu-view-wcappointments-staffs.php' );
      break;
      
      case 'wcfm-appointments-staffs-manage':
        require_once( $WCFMu->library->views_path . 'wc_appointments/wcfmu-view-wcappointments-staffs-manage.php' );
      break;
      
      case 'wcfm-appointments-manual':
        require_once( $WCFMu->library->views_path . 'wc_appointments/wcfmu-view-wcappointments-manual.php' );
      break;
      
      case 'wcfm-appointments-calendar':
        require_once( $WCFMu->library->views_path . 'wc_appointments/wcfmu-view-wcappointments-calendar.php' );
      break;
      
      case 'wcfm-appointments-details':
        require_once( $WCFMu->library->views_path . 'wc_appointments/wcfmu-view-wcappointments-details.php' );
      break;
	  }
	}
	
	/**
   * WC Appointments Ajax Controllers
   */
  public function wca_ajax_controller() {
  	global $WCFM, $WCFMu;
  	
  	$controllers_path = $WCFMu->plugin_path . 'controllers/wc_appointments/';
  	
  	$controller = '';
  	if( isset( $_POST['controller'] ) ) {
  		$controller = $_POST['controller'];
  		
  		switch( $controller ) {
  			case 'wcfm-products-manage':
  				require_once( $controllers_path . 'wcfmu-controller-wcappointments-products-manage.php' );
					new WCFMu_WCAppointments_Products_Manage_Controller();
				break;
				
				case 'wcfm-appointments':
					require_once( $controllers_path . 'wcfmu-controller-wcappointments.php' );
					new WCFMu_WCAppointments_Controller();
				break;
				
				case 'wcfm-appointments-staffs':
					require_once( $controllers_path . 'wcfmu-controller-wcappointments-staffs.php' );
					new WCFMu_WCAppointments_Staffs_Controller();
				break;
				
				case 'wcfm-appointments-staffs-manage':
					require_once( $controllers_path . 'wcfmu-controller-wcappointments-staffs-manage.php' );
					new WCFMu_WCAppointments_Staffs_Manage_Controller();
				break;
  		}
  	}
  }
  
  /**
   * WC Appointments Product General Options
   */
  function wca_product_manage_general( $product_id, $product_type ) {
  	global $WCFM, $WCFMu;
  	
  	require_once( $WCFMu->library->views_path . 'wc_appointments/wcfmu-view-wcappointments-products-manage.php' );
  }
  
  	/**
   * Handle Appointment confirmation
   */
  public function wcfm_appointment_mark_confirm() {
  	global $WCFM, $WCFMu;
  	
  	$appointment_id = $_POST['appointmentid'];
  	
  	$appointment = get_wc_appointment( $appointment_id );
		if ( 'confirmed' !== $appointment->get_status() ) {
			$appointment->update_status( 'confirmed' );
		}
		die;
  }
  
  /**
   * Handle Appointment Details Status Update
   */
  public function wcfm_modify_appointment_status() {
  	global $WCFM, $WCFMu;
  	
  	$appointment_id = $_POST['appointment_id'];
  	$appointment_status = $_POST['appointment_status'];
  	
  	$appointment = get_wc_appointment( $appointment_id );
  	$appointment->update_status( $appointment_status );
  	
		die;
  }
  
  function wcfm_before_appointment_object_save( $wcappointment, $wcappointment_data_store ) {
  	if( wcfm_is_vendor()) {
  		$wcappointment->set_customer_id( 0 );
  	}
  }
}